/* Tabela ITCOMISSAO*/
/* Descrio: A trigger insere registros na tabela LOGOPERA a partir de alteraes na tabela ITCOMISSAO*/
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela ITCOMISSAO, utilizando a tabela Geral de Logs de Operaes */

IF EXISTS(SELECT *
          FROM SYSOBJECTS O, SYSUSERS U
          WHERE O.NAME = 'LOG_EXP_ITCOMISSAO' AND
                TYPE   = 'TR'              AND
                U.UID  = O.UID             AND
                U.NAME = USER)

DROP TRIGGER LOG_EXP_ITCOMISSAO
GO
CREATE TRIGGER LOG_EXP_ITCOMISSAO
ON ITCOMISSAO
FOR UPDATE,INSERT, DELETE
AS
BEGIN
  DECLARE @V_USUARIO             VARCHAR(40);
  DECLARE @V_TIPO                VARCHAR(1);
  DECLARE @V_TPMOVI              VARCHAR(1);
  DECLARE @V_ITPESQ              VARCHAR(40);
  DECLARE @V_DESC                VARCHAR(500);
  DECLARE @V_CDFILLOGOPER        VARCHAR(4);
  DECLARE @V_DSREMOTEADDR        VARCHAR(15);
  DECLARE @V_DSREMOTEHOST        VARCHAR(15);
  DECLARE @V_DSSERVERRDM         VARCHAR(20);
  DECLARE @V_DSSERVMETHOD        VARCHAR(30);
  DECLARE @V_DSREMOTEOPER        VARCHAR(12);
  DECLARE @V_DTOPER              DATETIME;

  DECLARE @V_CDFILIAL_NEW        VARCHAR(4);
  DECLARE @V_CDCOMISSAO_NEW      VARCHAR(10);
  DECLARE @V_CDPRODUTO_NEW       VARCHAR(10);
  DECLARE @V_VRINICVENDA_NEW     DOUBLE PRECISION;
  DECLARE @V_VRFINAVENDA_NEW     DOUBLE PRECISION;
  DECLARE @V_VRPECOMIVEND_NEW    DOUBLE PRECISION;

  DECLARE @V_CDFILIAL_OLD        VARCHAR(4);
  DECLARE @V_CDCOMISSAO_OLD      VARCHAR(10);
  DECLARE @V_CDPRODUTO_OLD       VARCHAR(10);
  DECLARE @V_VRINICVENDA_OLD     DOUBLE PRECISION;
  DECLARE @V_VRFINAVENDA_OLD     DOUBLE PRECISION;
  DECLARE @V_VRPECOMIVEND_OLD    DOUBLE PRECISION;


   SET @V_TIPO = '0';
   SET @V_DESC = '';
   SET @V_CDFILLOGOPER = '';
   SET @V_USUARIO = '-';

   DECLARE C_CURSOR_R CURSOR FOR
      SELECT
         'D' IDCOMANDO,
         NULL, NULL, NULL, NULL, NULL,
         NULL,
         CDFILIAL,
         CDCOMISSAO,
         CDPRODUTO,
         VRINICVENDA,
         VRFINAVENDA,
         VRPECOMIVEND
      FROM
         DELETED
      WHERE
         0 = (SELECT COUNT(*) FROM INSERTED)

      UNION ALL

      SELECT
         'U' IDCOMANDO,
         INSERTED.CDFILIAL,
         INSERTED.CDCOMISSAO,
         INSERTED.CDPRODUTO,
         INSERTED.VRINICVENDA,
         INSERTED.VRFINAVENDA,
         INSERTED.VRPECOMIVEND,

         DELETED.CDFILIAL,
         DELETED.CDCOMISSAO,
         DELETED.CDPRODUTO,
         DELETED.VRINICVENDA,
         DELETED.VRFINAVENDA,
         DELETED.VRPECOMIVEND

      FROM
         INSERTED,
         DELETED
      WHERE
         INSERTED.CDFILIAL    = DELETED.CDFILIAL   AND
         INSERTED.CDCOMISSAO  = DELETED.CDCOMISSAO AND
         INSERTED.CDPRODUTO   = DELETED.CDPRODUTO  AND
         INSERTED.VRINICVENDA = DELETED.VRINICVENDA

      UNION ALL

      SELECT
         'I' IDCOMANDO,
         INSERTED.CDFILIAL,
         INSERTED.CDCOMISSAO,
         INSERTED.CDPRODUTO,
         INSERTED.VRINICVENDA,
         INSERTED.VRFINAVENDA,
         INSERTED.VRPECOMIVEND,

         NULL, NULL, NULL, NULL, NULL,
         NULL
      FROM
         INSERTED
      WHERE
         0 = (SELECT COUNT(*) FROM DELETED)

   OPEN C_CURSOR_R;
   FETCH NEXT FROM C_CURSOR_R INTO @V_TIPO,
                                   @V_CDFILIAL_NEW, @V_CDCOMISSAO_NEW, @V_CDPRODUTO_NEW, @V_VRINICVENDA_NEW, @V_VRFINAVENDA_NEW, @V_VRPECOMIVEND_NEW,

                                   @V_CDFILIAL_OLD, @V_CDCOMISSAO_OLD, @V_CDPRODUTO_OLD, @V_VRINICVENDA_OLD, @V_VRFINAVENDA_OLD, @V_VRPECOMIVEND_OLD;

   WHILE (@@FETCH_STATUS = 0) BEGIN

      IF @V_TIPO = 'D' BEGIN

         IF @V_CDFILIAL_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Filial = ' + ISNULL(@V_CDFILIAL_OLD,'') + ' ';
         END;
         IF @V_CDCOMISSAO_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Tabela de comisso = ' + ISNULL(@V_CDCOMISSAO_OLD,'') + ' ';
         END;
         IF @V_CDPRODUTO_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Produto = ' + ISNULL(@V_CDPRODUTO_OLD,'') + ' ';
         END;
         IF @V_VRINICVENDA_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Vr. Inicial = ' + ISNULL(CONVERT(VARCHAR,@V_VRINICVENDA_OLD),'') + ' ';
         END;
         IF @V_VRFINAVENDA_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Vr. Final = ' + ISNULL(CONVERT(VARCHAR,@V_VRFINAVENDA_OLD),'') + ' ';
         END;
         IF @V_VRPECOMIVEND_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Comisso = ' + ISNULL(CONVERT(VARCHAR,@V_VRPECOMIVEND_OLD),'') + '% ';
         END;

         SET @V_TPMOVI = '3';
         SET @V_ITPESQ = @V_CDFILIAL_OLD + @V_CDCOMISSAO_OLD + @V_CDPRODUTO_OLD + @V_VRINICVENDA_OLD;
      END
      ELSE BEGIN
         IF @V_TIPO = 'U' BEGIN
             IF (@V_CDFILIAL_OLD     <> @V_CDFILIAL_NEW    ) OR
                (@V_CDCOMISSAO_OLD   <> @V_CDCOMISSAO_NEW  ) OR
                (@V_CDPRODUTO_OLD    <> @V_CDPRODUTO_NEW   ) OR
                (@V_VRINICVENDA_OLD  <> @V_VRINICVENDA_NEW ) OR
                (@V_VRFINAVENDA_OLD  <> @V_VRFINAVENDA_NEW ) OR
                (@V_VRPECOMIVEND_OLD <> @V_VRPECOMIVEND_NEW) BEGIN

               SET @V_TPMOVI = '2';
               SET @V_ITPESQ = @V_CDFILIAL_OLD + @V_CDCOMISSAO_OLD + @V_CDPRODUTO_OLD + @V_VRINICVENDA_OLD;
            END;
         END
         ELSE BEGIN
            SET @V_TPMOVI = '1';
            SET @V_ITPESQ = @V_CDFILIAL_NEW + @V_CDCOMISSAO_NEW + @V_CDPRODUTO_NEW + @V_VRINICVENDA_NEW;
         END;

         IF ISNULL(@V_CDFILIAL_NEW, '!') <> ISNULL(@V_CDFILIAL_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + 'Filial = ' + @V_CDFILIAL_NEW;
         END;
         IF ISNULL(@V_CDCOMISSAO_NEW, '!') <> ISNULL(@V_CDCOMISSAO_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + 'Tabela de comisso = ' + @V_CDCOMISSAO_NEW;
         END;
         IF ISNULL(@V_CDPRODUTO_NEW, '!') <> ISNULL(@V_CDPRODUTO_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + 'Produto = ' + @V_CDPRODUTO_NEW;
         END;
         IF ISNULL(CONVERT(VARCHAR,@V_VRINICVENDA_NEW), '!') <> ISNULL(CONVERT(VARCHAR,@V_VRINICVENDA_OLD), '!') BEGIN
            SET @V_DESC = @V_DESC + 'Vr. Inicial = ' + CONVERT(VARCHAR,@V_VRINICVENDA_NEW);
         END;
         IF ISNULL(CONVERT(VARCHAR,@V_VRFINAVENDA_NEW), '!') <> ISNULL(CONVERT(VARCHAR,@V_VRFINAVENDA_OLD), '!') BEGIN
            SET @V_DESC = @V_DESC + 'Vr. Final = ' + CONVERT(VARCHAR,@V_VRFINAVENDA_NEW);
         END;
         IF ISNULL(CONVERT(VARCHAR,@V_VRPECOMIVEND_NEW), '!') <> ISNULL(CONVERT(VARCHAR,@V_VRPECOMIVEND_OLD), '!') BEGIN
            SET @V_DESC = @V_DESC + 'Comisso = ' + CONVERT(VARCHAR,@V_VRPECOMIVEND_NEW);
         END;
      END;

      IF @V_TPMOVI <> '0' BEGIN
         EXEC NMUSUARIO  @V_USUARIO;
         EXEC WEBTUNNING @V_DSREMOTEADDR,
                         @V_DSREMOTEHOST,
                         @V_DSSERVERRDM,
                         @V_DSSERVMETHOD,
                         @V_DSREMOTEOPER;
         IF NOT @V_DESC IS NULL BEGIN

            SET @V_DTOPER = GETDATE();

            INSERT INTO LOGOPERA
               (NMTABELA,
                DTOPERACAO,
                NMRESPOPER,
                IDTIPOOPER,
                CDPESQITEM,
                DSOPERACAO,
                CDFILLOGOPER,
                DSRDMUSUOPER,
                DSENDUSUOPER,
                DSNMUSUOPER,
                DSMETUSUOPER,
                CDORESPOPER)
            VALUES
               ('ITCOMISSAO',
                @V_DTOPER,
                @V_USUARIO,
                @V_TPMOVI,
                @V_ITPESQ,
                SUBSTRING(@V_DESC,1,300),
                @V_CDFILLOGOPER,
                @V_DSSERVERRDM,
                @V_DSREMOTEADDR,
                @V_DSREMOTEHOST,
                @V_DSSERVMETHOD,
                @V_DSREMOTEOPER);

         END;
      END;
      FETCH NEXT FROM C_CURSOR_R INTO @V_TIPO,
                                      @V_CDFILIAL_NEW, @V_CDCOMISSAO_NEW, @V_CDPRODUTO_NEW, @V_VRINICVENDA_NEW, @V_VRFINAVENDA_NEW, @V_VRPECOMIVEND_NEW,

                                      @V_CDFILIAL_OLD, @V_CDCOMISSAO_OLD, @V_CDPRODUTO_OLD, @V_VRINICVENDA_OLD, @V_VRFINAVENDA_OLD, @V_VRPECOMIVEND_OLD;
   END;
   CLOSE C_CURSOR_R;
   DEALLOCATE C_CURSOR_R;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'LOG_EXP_ITCOMISSAO','5.85.XXX';
GO